"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _EventHandler = _interopRequireDefault(require("./EventHandler"));

var _uniqueID = _interopRequireDefault(require("../uniqueid/uniqueID"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Event = /*#__PURE__*/function () {
  function Event() {
    _classCallCheck(this, Event);

    _defineProperty(this, "_handlers", void 0);

    this._handlers = {};
  }
  /**
   * Add a subscription to this event
   *
   * @param  callback  run when the event is triggered.
   * @return this
   */


  _createClass(Event, [{
    key: "addHandler",
    value: function addHandler(callback) {
      var key = (0, _uniqueID.default)();
      this._handlers[key] = callback;
      return new _EventHandler.default(this, key);
    }
    /**
     * Destroys this event. Removes all handlers.
     *
     * @return this
     */

  }, {
    key: "remove",
    value: function remove() {
      this._handlers = {};
      return this;
    }
    /**
     * Removes a subscription by key.
     *
     * @param  key   id of the subscription to remove
     * @return this
     */

  }, {
    key: "removeHandler",
    value: function removeHandler(key) {
      delete this._handlers[key];
      return this;
    }
    /**
     * @protected
     * Run a handler by key if it exists
     *
     * @param  key  id of the handler to run
     */

  }, {
    key: "_runHandler",
    value: function _runHandler(key) {
      if (this._handlers.hasOwnProperty(key)) {
        this._handlers[key].call(null);
      }
    }
    /**
     * Run all subscribed handlers.
     *
     * @return this
     */

  }, {
    key: "runHandlers",
    value: function runHandlers() {
      Object.keys(this._handlers).forEach(this._runHandler.bind(this));
      return this;
    }
    /**
     * Convenience method for running multiple events.
     *
     * @param  events  a list of events to run.
     */

  }], [{
    key: "runMultiple",
    value: function runMultiple(events) {
      events.forEach(function (evt) {
        return evt.runHandlers();
      });
    }
  }]);

  return Event;
}();

exports.default = Event;